'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var view = require('@codemirror/view');
var highlight = require('@codemirror/highlight');

const highlightBackground = '#2c313a', selection = '#3E4451';
const nord0 = '#2e3440', nord3 = '#4c566a', nord4 = '#d8dee9', nord5 = '#e5e9f0', nord7 = '#8fbcbb', nord8 = '#88c0d0', nord9 = '#81a1c1', nord11 = '#bf616a', nord12 = '#d08770', nord14 = '#a3be8c', nord15 = '#b48ead';
/**
The editor theme styles for Nord.
*/
const nordTheme = view.EditorView.theme({
    '&': {
        color: nord4,
        backgroundColor: nord0,
    },
    '.cm-content': {
        caretColor: nord4,
    },
    '&.cm-focused .cm-cursor': { borderLeftColor: nord4 },
    '&.cm-focused .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: selection },
    '.cm-panels': { backgroundColor: nord0, color: nord4 },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: '#72a1ff59',
        outline: '1px solid #457dff',
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: '#6199ff2f',
    },
    '.cm-activeLine': { backgroundColor: highlightBackground },
    '.cm-selectionMatch': { backgroundColor: '#aafe661a' },
    '.cm-matchingBracket, .cm-nonmatchingBracket': {
        backgroundColor: '#bad0f847',
        outline: '1px solid #515a6b',
    },
    '.cm-gutters': {
        backgroundColor: nord0,
        color: nord3,
        border: 'none',
    },
    '.cm-activeLineGutter': {
        backgroundColor: highlightBackground,
    },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: '#ddd',
    },
    '.cm-tooltip': {
        border: '1px solid #181a1f',
        backgroundColor: nord0,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            backgroundColor: highlightBackground,
            color: nord4,
        },
    },
}, { dark: true });
/**
The highlighting style for code in the One Dark theme.
*/
const nordHighlightStyle = highlight.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: nord9 },
    {
        tag: [highlight.tags.deleted, highlight.tags.character, highlight.tags.propertyName, highlight.tags.macroName],
        color: nord7,
    },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: nord8 },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: nord4 },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: nord4 },
    {
        tag: [
            highlight.tags.typeName,
            highlight.tags.className,
            highlight.tags.changed,
            highlight.tags.annotation,
            highlight.tags.modifier,
            highlight.tags.self,
            highlight.tags.namespace,
        ],
        color: nord12,
    },
    {
        tag: [highlight.tags.typeName],
        color: nord9,
    },
    { tag: highlight.tags.number, color: nord15 },
    {
        tag: [highlight.tags.url, highlight.tags.link],
        color: nord8,
    },
    {
        tag: [highlight.tags.operator, highlight.tags.operatorKeyword],
        color: nord9,
    },
    { tag: [highlight.tags.escape, highlight.tags.regexp, highlight.tags.special(highlight.tags.string)], color: nord14 },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: nord3 },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
    { tag: highlight.tags.link, color: nord3, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: nord7 },
    { tag: [highlight.tags.bool], color: nord9 },
    { tag: [highlight.tags.special(highlight.tags.variableName)], color: nord5 },
    { tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted], color: nord14 },
    { tag: highlight.tags.invalid, color: nord11 },
]);
/**
Extension to enable the One Dark theme (both the editor theme and
the highlight style).
*/
const nord = [nordTheme, nordHighlightStyle];

exports.nord = nord;
exports.nordHighlightStyle = nordHighlightStyle;
exports.nordTheme = nordTheme;
